<?php

namespace Drupal\comment_auto_approve;

use Drupal\comment\CommentInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\user\UserInterface;

/**
 * Comment Auto Approve handling helper service.
 */
class CommentAutoApprove implements CommentAutoApproveInterface {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new CommentAutoApprove object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function preProcessComment(CommentInterface $comment) {

    if (!$comment->isPublished()) {
      $owner = $comment->getOwner();

      if ($owner instanceof UserInterface && !$owner->hasPermission('skip comment approval') && $this->isToApprove($comment)) {
        $comment->setPublished();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isToApprove(CommentInterface $comment) {
    $commented_entity = $comment->getCommentedEntity();

    if ($commented_entity instanceof FieldableEntityInterface) {
      $comment_field_definition = $commented_entity->getFieldDefinition($comment->getFieldName());

      if ($comment_field_definition instanceof FieldDefinitionInterface) {
        $comment_field_config = $comment_field_definition->getConfig($commented_entity->bundle());

        if ($comment_field_config instanceof FieldConfigInterface && $comment_field_config->getThirdPartySetting('comment_auto_approve', 'enabled')) {
          $limit = (int) $comment_field_config->getThirdPartySetting('comment_auto_approve', 'manual_approve_qty');
          $logic = $comment_field_config->getThirdPartySetting('comment_auto_approve', 'approved_count_logic');

          $query = $this->entityTypeManager->getStorage($comment->getEntityTypeId())->getQuery();
          $query->condition('uid', $comment->getOwnerId());
          $query->condition('status', CommentInterface::PUBLISHED);

          if ($logic === CommentAutoApproveInterface::LIMIT_LOCAL) {
            $query->condition('field_name', $comment->getFieldName());
            $query->condition('entity_type', $comment->getCommentedEntityTypeId());
          }

          $query->range(0, $limit);
          $result = $query->execute();

          return count($result) >= $limit;
        }
      }
    }

    return FALSE;
  }

}
